/**
 * \file output/xpip.cpp
 * \date 13/4/2017
 * \author Olivier Langella
 * \brief XPIP writer
 */

/*******************************************************************************
* Copyright (c) 2017 Olivier Langella <olivier.langella@u-psud.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <olivier.langella@u-psud.fr> - initial API and implementation
******************************************************************************/

#include "xpip.h"
#include "../config.h"
#include <QDateTime>
#include <pappsomspp/pappsoexception.h>

Xpip::Xpip(const QString & out_filename)
{
    //_p_digestion_pipeline = p_digestion_pipeline;

    //_mzidentml = "http://psidev.info/psi/pi/mzIdentML/1.1";
    QString complete_out_filename = out_filename;
    _output_file = new QFile(complete_out_filename);

    if (_output_file->open(QIODevice::WriteOnly))
    {
        _output_stream = new QXmlStreamWriter();
        _output_stream->setDevice(_output_file);
    } else
    {
        throw pappso::PappsoException(QObject::tr("error : cannot open the XPIP output file : %1\n").arg(out_filename));
    }

    _output_stream->setAutoFormatting(true);
    _output_stream->writeStartDocument("1.0");


}

Xpip::~Xpip()
{
    delete _output_file;
    delete _output_stream;
}

void Xpip::close() {
    _output_stream->writeEndDocument();
    _output_file->close();
}

void Xpip::write(ProjectSp sp_project) {

    _sp_project = sp_project;
    if (_sp_project.get() == nullptr) {
        throw pappso::PappsoException(QObject::tr("Error writing XPIP :\n project is empty"));
    }

//<xtandem_pipeline version="3.3.5">
    _output_stream->writeStartElement("xpip");
    _output_stream->writeAttribute("version","4.0");
    //_output_stream->writeAttribute("creationDate", QDateTime::currentDateTime().toString( Qt::ISODate));
    _output_stream->writeNamespace("http://www.w3.org/2001/XMLSchema-instance","xsi");
    //_output_stream->writeNamespace("http://www.w3.org/2001/XMLSchema-instance","xsi");
    _output_stream->writeAttribute("xmlns","http://pappso.inra.fr/xsd/xpip/4.0");
    //xsi:schemaLocation="http://psidev.info/psi/pi/mzIdentML/1.1 http://www.psidev.info/files/mzIdentML1.1.0.xsd"
    _output_stream->writeAttribute("http://www.w3.org/2001/XMLSchema-instance","schemaLocation","http://pappso.inra.fr/xsd/xpip/4.0 http://pappso.inra.fr/xsd/xpip-4.0.xsd");


    //<information Data_Type="combi" match_number="6807"/>
    // _output_stream->writeStartElement("information");
    //  _output_stream->writeAttribute("Data_Type","combi");
    // _output_stream->writeEndElement();
    writeDescription();

    //<filter_params pep_evalue="0.01" prot_evalue="-2.0" pep_number="2" filter_to_all="false" database_filter="/gorgone/pappso/jouy/raw/2013_Orbitrap/2013_07_04_Chanat/contaminants_standarts.fasta"/>
    //const AutomaticFilterParameters & getAutomaticFilterParameters() const;
    writeFilterParameters(_sp_project.get()->getAutomaticFilterParameters());

    writeMsrunList(_sp_project.get()->getMsRunStore());
    writeIdentificationDataSourceList(_sp_project.get()->getIdentificationDataSourceStore());
    /*
        <identifications>
        <identification>
        <modifs_list_mass>
        <modifs_mass modvalue="-18.01056"/>
                              <modifs_mass modvalue="-17.02655"/>
                                      <modifs_mass modvalue="15.99491"/>
                                              <modifs_mass modvalue="42.01057"/>
                                                      <modifs_mass modvalue="42.01056"/>
                                                              <modifs_mass modvalue="57.02146"/>
                                                                      </modifs_list_mass>
                                                                      <samples>
                                                                      <sample value="20120906_balliau_extract_1_A01_urnb-1"/>
                                                                              <sample value="20120906_balliau_extract_1_A02_urzb-1"/>
                                                                                      </samples>
                                                                                      //<masschroq>
                                                                                      */
    // <rawdata><!-- time_values_dir="directory" to read retention time corrections-->
    _output_stream->writeStartElement("rawdata");
    _output_stream->writeComment("time_values_dir=\"directory\" to read retention time corrections");

    std::vector<MsRunSp> msrun_list = _sp_project.get()->getMsRunStore().getMsRunList();
    for (MsRunSp & msrun : msrun_list) {

        //<data_file id="samp0" format="mzxml" path="bsa1.mzXML" type="centroid" />
        _output_stream->writeStartElement("data_file");
        _output_stream->writeAttribute("id",msrun.get()->getXmlId());
        if(msrun.get()->getMzFormat() == MzFormat::mzXML) {
            _output_stream->writeAttribute("format","mzxml");
        }
        _output_stream->writeAttribute("path",msrun.get()->getFilename());
        _output_stream->writeAttribute("type","centroid");
        // <data_file id="samp1" format="mzxml" path="bsa2.mzXML" type="profile" />
        _output_stream->writeEndElement();
    }
    // <data_file id="samp2" format="mzml" path="/home/user/bsa3.mzml"      type="profile" />
//<data_file id="samp3" format="mzml" path="/home/user/bsa4.mzml"             type="profile" />
    // _output_stream.writeEndElement();
    //             </rawdata>
    _output_stream->writeEndElement();


    _output_stream->writeEndDocument();
}
void Xpip::writeDoubleAttribute(const QString & attribute, pappso::pappso_double value) {
    _output_stream->writeAttribute(attribute,QString::number(value, 'f', 10));

}
void Xpip::writeBooleanAttribute(const QString & attribute, bool value) {
    if (value) {
        _output_stream->writeAttribute(attribute,"true");
    }
    else {
        _output_stream->writeAttribute(attribute,"false");
    }

}
void Xpip::writeFilterParameters(const AutomaticFilterParameters & filters) {

    _output_stream->writeStartElement("filter_params");
    writeDoubleAttribute("pep_evalue",filters.getFilterPeptideEvalue());
    writeDoubleAttribute("prot_evalue",filters.getFilterProteinEvalue());
    _output_stream->writeAttribute("pep_number",QString("%1").arg(filters.getFilterMinimumPeptidePerMatch()));
    writeBooleanAttribute("cross_sample",filters.getFilterCrossSamplePeptideNumber());
    _output_stream->writeEndElement();
}

void Xpip::writeDescription() {

    _output_stream->writeStartElement("description");
    _output_stream->writeAttribute("version",XTPCPP_VERSION);
    writeBooleanAttribute("combine",_sp_project.get()->isCombineMode());
    _output_stream->writeAttribute("date",QDateTime::currentDateTime().toString( Qt::ISODate));
    _output_stream->writeEndElement();
}


void Xpip::writeIdentificationDataSourceList(const IdentificationDataSourceStore & ident_store) {

    _output_stream->writeStartElement("identification_source_list");
    for (const IdentificationDataSourceSp ident_source_sp : ident_store.getIdentificationDataSourceList()) {
        _output_stream->writeStartElement("identification_source");
        _output_stream->writeAttribute("id",ident_source_sp.get()->getXmlId());
        _output_stream->writeAttribute("msrun_id",ident_source_sp.get()->getMsRunSp().get()->getXmlId());
        //_output_stream->writeAttribute("format",QString("%1").arg(static_cast<std::int8_t>(ident_source_sp.get()->getIdentFormat())));
        _output_stream->writeAttribute("path",ident_source_sp.get()->getResourceName());

        _output_stream->writeAttribute("engine",QString("%1").arg(static_cast<std::int8_t>(ident_source_sp.get()->getIdentificationEngine())));
        _output_stream->writeAttribute("version",ident_source_sp.get()->getIdentificationEngineVersion());

        _output_stream->writeEndElement();
    }
    _output_stream->writeEndElement();

}

void Xpip::writeMsrunList(const MsRunStore & msrun_store) {

    _output_stream->writeStartElement("msrun_list");
    for (const MsRunSp msrun_sp : msrun_store.getMsRunList()) {
        _output_stream->writeStartElement("msrun");
        _output_stream->writeAttribute("id",msrun_sp.get()->getXmlId());
        _output_stream->writeAttribute("name",msrun_sp.get()->getSampleName());
        _output_stream->writeAttribute("format",QString("%1").arg(static_cast<std::int8_t>(msrun_sp.get()->getMzFormat())));
        QFileInfo mz_info(msrun_sp.get()->getFilename());
        if (mz_info.exists()) {
            _output_stream->writeAttribute("path",msrun_sp.get()->getFilename());
        }
        else {
            _output_stream->writeAttribute("path",mz_info.completeBaseName());
        }
        const std::map<MsRunStatistics, QVariant> stats = msrun_sp.get()->getMsRunStatisticsMap();
        if (stats.size() > 0) {
            _output_stream->writeStartElement("stats");
            for (const std::pair<MsRunStatistics, QVariant> stat_pair: stats) {
                _output_stream->writeStartElement("stat");
                _output_stream->writeAttribute("key",QString("%1").arg(static_cast<std::int8_t>(stat_pair.first)));
                _output_stream->writeAttribute("value",stat_pair.second.toString());
                _output_stream->writeEndElement();
            }
            _output_stream->writeEndElement();
        }
        _output_stream->writeEndElement();
    }
    _output_stream->writeEndElement();
}
